/*
 * Copyright (c) 2003, 2004 Henri Sivonen, Yrj Kari-Koskinen
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 */

package fi.iki.hsivonen.util;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CodingErrorAction;
import java.util.Arrays;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import javax.xml.parsers.DocumentBuilder;

import multivalent.std.adaptor.pdf.Dict;
import multivalent.std.adaptor.pdf.PDFReader;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import fi.iki.hsivonen.gnu.xml.pipeline.DomConsumer;
import fi.iki.hsivonen.xml.DOMUtils;
import fi.iki.hsivonen.xml.SAXUtils;

/**
 *
 * @author hsivonen
 * @author ykk
 */
public class MetadataExtractor {
	/** Do we print the debug information */
	private static boolean DEBUG = false;

	private static Pattern charsetPat =
		Pattern.compile("^.*charset\\s*=\\s*((?:-|\\w)+).*$");

	private File file;
	private String contentType;
	private FileInputStream stream;
	private String title;
	private Document doc;
	private String encoding;
	private String fileName;
	private boolean hasCSSCharset = false;
	private boolean guessedContentType = false;

	private static final char ANY = '\uE000';
	private static final char DIGIT = '\uE001';

	private static final int MAX_MAGIC_LENGTH = 12;

	private static char[][] magicNumbers = {
		{'\u0089', 'P', 'N', 'G', '\r', '\n', '\u001A', '\n'}, // PNG 1.0 spec
		{'\u008a', 'M', 'N', 'G', '\r', '\n', '\u001A', '\n'}, // MNG spec
		{'G', 'I', 'F', '8', '7', 'a'}, // BSD /etc/magic
		{'G', 'I', 'F', '8', '9', 'a'}, // BSD /etc/magic
		{'%', 'P', 'D', 'F', '-', DIGIT, '.', DIGIT},
		{'%', '!'},
		{'P', 'K', '\u0003', '\u0004'},
		{'F', 'W', 'S'}, // observation and BSD /etc/magic
		{'.', 'R', 'M', 'F', '\u0000'}, // observation
		{'{', '\\', 'r', 't', 'f'},
		{'\u0000', '\u0000', '\u0001', '\u00BA'},
		{'\u0000', '\u0000', '\u0001', '\u00B3'},
		{ANY, ANY, ANY, ANY, ANY, ANY, 'J', 'F', 'I', 'F', '\u0000'}, // observation, BSD /etc/magic and the JFIF spec http://www.w3.org/Graphics/JPEG/jfif3.pdf
		{ANY, ANY, ANY, ANY, 'm', 'o', 'o', 'v'}, // /usr/share/misc/file/magic
		{ANY, ANY, ANY, ANY, 'm', 'd', 'a', 't'}, // /usr/share/misc/file/magic
		{ANY, ANY, ANY, ANY, 'f', 'r', 'e', 'e'}, // /usr/share/misc/file/magic
		{ANY, ANY, ANY, ANY, 'j', 'u', 'n', 'k'}, // /usr/share/misc/file/magic
		{ANY, ANY, ANY, ANY, 'p', 'n', 'o', 't'}, // /usr/share/misc/file/magic
		{ANY, ANY, ANY, ANY, 's', 'k', 'i', 'p'}, // /usr/share/misc/file/magic
		{ANY, ANY, ANY, ANY, 'w', 'i', 'd', 'e'}, // /usr/share/misc/file/magic
		{ANY, ANY, ANY, ANY, 'p', 'i', 'c', 't'}, // /usr/share/misc/file/magic
		{ANY, ANY, ANY, ANY, 'f', 't', 'y', 'p'}, // observation
		{'R', 'I', 'F', 'F', ANY, ANY, ANY, ANY, 'A', 'V', 'I'}, // /usr/share/file/magic.mime
		{'R', 'I', 'F', 'F', ANY, ANY, ANY, ANY, 'W', 'A', 'V', 'E'}, // /usr/share/file/magic.mime
		{'\u00FF', '\u00FA'}, // /usr/share/file/magic.mime
		{'\u00FF', '\u00FB'}, // /usr/share/file/magic.mime
		{'I', 'D', '3'}, // /usr/share/file/magic.mime
		{'O', 'g', 'g', 'S'}, // /usr/share/file/magic.mime
		{'B', 'M'}, // /usr/share/file/magic.mime
		{'S', 't', 'u', 'f', 'f', 'I', 't'}, // observation
		{'S', 'I', 'T', '!'}, // observation
		{'\u00FE', '\u00FF'}, // big endian BOM
		{'\u00FF', '\u00FE'}, // little endian BOM
		{'@', 'c', 'h', 'a', 'r', 's', 'e', 't'},
		{'I', 'I'},
		{'M', 'M'},
		{'\u0000', '\u0000', '\u0000', '\u000C', '\u006A', '\u0050', '\u0020', '\u0020', '\r', '\n', '\u0087', '\n'} // http://www.iana.org/assignments/media-types/image/jp2
	};


    private static String[] magicTypes =
        {
            "image/png",
            "video/x-mng",
            "image/gif",
            "image/gif",
            "application/pdf",
            "application/postscript",
            "application/zip",
            "application/x-shockwave-flash",
            "audio/x-pn-realaudio",
            "text/rtf",
            "video/mpeg",
            "video/mpeg",
            "image/jpeg",
            "video/quicktime",
            "video/quicktime",
            "video/quicktime",
            "video/quicktime",
            "video/quicktime",
            "video/quicktime",
            "video/quicktime",
            "video/quicktime",
            "video/mp4",
            "video/x-msvideo",
            "audio/x-wav",
            "audio/mpeg",
            "audio/mpeg",
            "audio/mpeg",
            "application/ogg",
            "image/x-bmp",
            "application/x-stuffit",
            "application/x-stuffit",
            "utf-16",
        // see determineTypeFromMagic
        "utf-16", // see determineTypeFromMagic
        "CSS_CHARSET", // see determineTypeFromMagic
        "image/tiff", "image/tiff", "image/jp2" };

    // must be sorted lexicographically
    private static String[] extensions =
        {
            "asf",
            "css",
            "doc",
            "htm",
            "html",
            "ppt",
            "txt",
            "wma",
            "wmv",
            "xls" };

    private static String[] extensionTypes =
        {
            "video/x-ms-asf",
            "text/css",
            "application/msword",
            "text/html",
            "text/html",
            "application/vnd.ms-powerpoint",
            "text/plain",
            "audio/x-ms-wma",
            "video/x-ms-wmv",
            "application/vnd.ms-excel" };

    private static String[] mimeTypes = {
        // teksti
        "text/plain",
            "text/html",
            "application/xhtml+xml",
            "application/xml",
            "text/css",
            "text/rtf",
            "application/pdf",
            "application/postscript",

        // toimisto-ohjelmat
        "application/msword",
            "application/vnd.ms-excel",
            "application/vnd.ms-powerpoint",
            "application/vnd.sun.xml.writer",
            "application/vnd.sun.xml.calc",
            "application/vnd.sun.xml.impress",
            "application/vnd.sun.xml.draw",

        // kuvat
        "image/jpeg",
            "image/gif",
            "image/png",
            "image/x-bmp",
            "image/svg+xml",
            "image/tiff",
            "image/jp2",

        // video
        "video/mpeg",
            "video/mp4",
            "video/x-msvideo",
            "video/x-ms-asf",
            "video/x-ms-wmv",
            "audio/x-pn-realaudio",
            "video/quicktime",
            "video/x-mng",

        // ni
        "audio/mpeg", "audio/x-wav", "audio/x-ms-wma", "application/ogg",

        // pakkaustykalut
        "application/zip",
            "application/x-stuffit",
            "application/x-shockwave-flash" };

    private static String[] mimeTypeNames = {
        // teksti
        "Tekstidokumentti",
            "HTML-dokumentti",
            "XHTML-dokumentti",
            "XML-dokumentti",
            "Tyylisivu (CSS)",
            "RTF-dokumentti",
            "PDF-dokumentti",
            "PostScript-dokumentti",

        // toimisto-ohjelmat
        "MSOffice Word -dokumentti",
            "MSOffice Excel -tykirja",
            "MSOffice Powerpoint -esitys",
            "OpenOffice Writer -dokumentti",
            "OpenOffice Calc -tykirja",
            "OpenOffice Impress -esitys",
            "OpenOffice Impress -piirros",

        // kuvat
        "JPEG-kuva",
            "GIF-kuva",
            "PNG-kuva",
            "BMP-kuva",
            "SVG-kuva",
            "TIFF-kuva",
            "JPEG 2000 -kuva",

        // video
        "MPEG-video",
            "MPEG4-video",
            "AVI-video",
            "ASF-video",
            "WMV-video",
            "Real Media -video",
            "Quicktime-video",
            "MNG-animaatio",

        // ni
        "MP3-ni", "WAV-ni", "WMA-ni", "OGG Vorbis -ni",

        // pakkaustykalut
        "ZIP-paketti", "StuffIt-paketti", "Flash-animaatio" };

    public static void main(String[] args) throws Exception {
        if (args.length < 1) {
            System.out.println("Usage: java MetadataExtractor filename");
            System.exit(0);
        }

        File f = new File(args[0]);
        MetadataExtractor me = new MetadataExtractor(f);
        System.out.println(
            "content-type:\t"
                + me.getContentType()
                + (me.getGuessedContentType() ? " (guessed)" : ""));
		System.out.println("encoding:\t\t" + me.getEncoding());
		System.out.println("title:\t\t" + me.getTitle());
        if (DEBUG && me.getTitle() != null) {
            for (int i = 0; i < me.getTitle().length(); i++) {
                System.out.println(
                    Integer.toHexString((int)me.getTitle().charAt(i)));
            }
        }
    }

    public MetadataExtractor(File f) throws IOException {
        this(f, null);
    }

    /** Creates a new instance of MetadataExtractor */
    public MetadataExtractor(File f, String fileName) throws IOException {
        DEBUG = (System.getProperty("DebugMetadataExtractor") != null);
        this.file = f;
        if (fileName == null) {
            this.fileName = f.getName();
        } else {
            this.fileName = fileName;
        }
        this.contentType = null;
        this.determineTypeFromMagic();
        if ("application/zip".equals(this.contentType)) {
            this.determineTypeFromZip();
        }
        if (this.contentType == null) {
            this.determineTypeFromXML();
        }
        if (this.contentType == null) {
            this.determineTypeFromExtension();
        }
        this.checkEncoding();
        if ("application/pdf".equals(this.contentType)) {
            this.extractMetaFromPDF();
        }
        if (this.contentType != null
            && this.contentType.startsWith("text/html")) {
            this.extractMetaFromHTML();
        }
    }

    private void checkEncoding() throws IOException {
        if (!("text/css".equals(this.contentType)
            || "text/html".equals(this.contentType)
            || "text/plain".equals(this.contentType))) {
            return;
        }
        if ("text/css".equals(this.contentType) && this.hasCSSCharset) {
            return;
        }
        if (this.encoding == null) {
            this.guessEncoding();
        }

        // ei haluta contentType-stringiin viel tss vaiheessa,
        //sill muuten kyttj ei voi valita contentType

        //this.contentType = this.contentType + "; charset=" + this.encoding;
    }

    private void determineTypeFromMagic() throws IOException {
        byte[] buf = new byte[MAX_MAGIC_LENGTH];
        FileInputStream stream = new FileInputStream(this.file);
        int numBytes = stream.read(buf);
        stream.close();

        if (DEBUG) {
            for (int i = 0; i < buf.length; i++) {
                System.out.println(Integer.toHexString(((int)buf[i]) & 0xFF));
            }
        }

        for (int i = 0; i < magicNumbers.length; i++) {
            boolean match = true;
            for (int j = 0; j < magicNumbers[i].length && j < numBytes; j++) {
                if (magicNumbers[i][j] == ANY) {
                    continue;
                } else if (
                    magicNumbers[i][j] == DIGIT
                        && buf[j] < '9'
                        && buf[j] > '0') {
                    continue;
                } else if (
                    magicNumbers[i][j] == (char) (((int)buf[j]) & 0xFF)) {
                    continue;
                } else {
                    match = false;
                    break;
                }
            }
            if (match) {
                if ("utf-16".equals(magicTypes[i])) {
                    this.encoding = "utf-16";
                } else if ("CSS_CHARSET".equals(magicTypes[i])) {
                    this.hasCSSCharset = true;
                } else {
                    this.contentType = magicTypes[i];
                }
                return;
            }
        }
    }

    private void guessEncoding() throws IOException {
        int b;
        boolean couldBeASCII = true;
        boolean couldBeISO = true;
        BufferedInputStream in =
            new BufferedInputStream(new FileInputStream(this.file));
        try {
            while (((b = in.read()) != -1) && (couldBeASCII || couldBeISO)) {
                if (b > 0x7F) {
                    couldBeASCII = false;
                    if (b < 0xA0) {
                        couldBeISO = false;
                    }
                }
            }
        } finally {
            in.close();
        }
        if (couldBeASCII) {
            this.encoding = "us-ascii";
            return;
        }
        if (this.couldBeUTF8()) {
            this.encoding = "utf-8";
            return;
        }
        if (couldBeISO) {
            this.encoding = "iso-8859-1";
            return;
        }
        this.encoding = "windows-1252";
        return;
    }

    private boolean couldBeUTF8() throws IOException {
        BufferedReader in;
        CharsetDecoder decoder = Charset.forName("UTF-8").newDecoder();
        decoder.onMalformedInput(CodingErrorAction.REPORT);
        decoder.onUnmappableCharacter(CodingErrorAction.REPORT);
        in =
            new BufferedReader(
                new InputStreamReader(new FileInputStream(this.file), decoder));
        try {
            while (in.read() != -1);
        } catch (CharacterCodingException e) {
            return false;
        } finally {
            in.close();
        }
        return true;
    }

    private void determineTypeFromXML() throws IOException {
        DocumentBuilder builder = DOMUtils.newNonvalidatingDocumentBuilder();
        // If the file can be parsed as XML without fatal errors, it is by
        // definition an XML document. OTOH, if there are fatal errors, it
        // by definition is not.
        try {
            InputSource is = new InputSource(new FileInputStream(this.file));
            is.setSystemId("file:///foo");
            this.doc = builder.parse(is);
        } catch (Exception e) {
            return;
        }
        // However, some HTML tag soup docs might by chance be well-formed.
        Element root = doc.getDocumentElement();
        if (root.getNamespaceURI() == null
            && "HTML".equalsIgnoreCase(root.getNodeName())) {
            this.contentType = "text/html";
            return;
        } else if (
            "http://www.w3.org/1999/xhtml".equals(root.getNamespaceURI())
                && "html".equals(root.getLocalName())) {
            loop : for (
                Node n = root.getFirstChild();
                    n != null;
                    n = n.getNextSibling()) {
                if ("http://www.w3.org/1999/xhtml".equals(n.getNamespaceURI())
                    && "head".equals(n.getLocalName())) {
                    for (Node m = n.getFirstChild();
                        m != null;
                        m = m.getNextSibling()) {
                        if ("http://www.w3.org/1999/xhtml"
                            .equals(m.getNamespaceURI())
                            && "title".equals(m.getLocalName())) {
                            this.setTitle(DOMUtils.textContent(m));
                            break loop;
                        }
                    }
                }
            }
            this.contentType = "application/xhtml+xml";
            return;
        } else if (
            "http://www.w3.org/2000/svg".equals(root.getNamespaceURI())
                && "svg".equals(root.getLocalName())) {
            for (Node n = root.getFirstChild();
                n != null;
                n = n.getNextSibling()) {
                if ("http://www.w3.org/2000/svg".equals(n.getNamespaceURI())
                    && "title".equals(n.getLocalName())) {
                    this.setTitle(DOMUtils.textContent(n));
                    break;
                }
            }
            this.contentType = "image/svg+xml";
            return;
        } else {
            this.contentType = "application/xml";
            return;
        }
    }

    private void determineTypeFromZip() throws IOException {
        ZipFile zf;
        try {
            zf = new ZipFile(this.file);
        } catch (ZipException e) {
            // doesn't look like a zip file after all
            this.contentType = null;
            return;
        }
        // no support for encrypted files
        Document metaDoc;
        DocumentBuilder builder = DOMUtils.newNonvalidatingDocumentBuilder();
        ZipEntry metaEntry;
        if ((metaEntry = zf.getEntry("meta.xml")) != null) {
            try {
                InputSource is = new InputSource(zf.getInputStream(metaEntry));
                is.setSystemId("file:///foo");
                metaDoc = builder.parse(is);
            } catch (Exception e) {
                return;
            }
            Node titleElt =
                DOMUtils.findElement(
                    metaDoc,
                    "http://purl.org/dc/elements/1.1/",
                    "title");
            if (titleElt != null) {
                this.setTitle(DOMUtils.textContent(titleElt));
            }
        }
        ZipEntry contentEntry;
        if ((contentEntry = zf.getEntry("content.xml")) == null) {
            // not an OOo file
            return;
        }
        try {
            InputSource is = new InputSource(zf.getInputStream(contentEntry));
            is.setSystemId("file:///foo");
            this.doc = builder.parse(is);
        } catch (Exception e) {
            return;
        }
        Element root = this.doc.getDocumentElement();
        if ("http://www.w3.org/1998/Math/MathML"
            .equals(root.getNamespaceURI())) {
            this.contentType = "application/vnd.sun.xml.math";
            return;
        }
        String docClass =
            root.getAttributeNS("http://openoffice.org/2000/office", "class");
        if ("text".equals(docClass)) {
            this.contentType = "application/vnd.sun.xml.writer";
            // If we didn't get a title from the metadata, 
            // let's use the first heading
            if (this.title == null) {
                Node headingElt =
                    DOMUtils.findElement(
                        root,
                        "http://openoffice.org/2000/text",
                        "h");
                if (headingElt != null) {
                    this.setTitle(DOMUtils.textContent(headingElt));
                }
            }
            return;
        } else if ("text-global".equals(docClass)) {
            this.contentType = "application/vnd.sun.xml.writer.global";
            return;
        } else if ("spreadsheet".equals(docClass)) {
            this.contentType = "application/vnd.sun.xml.calc";
            return;
        } else if ("drawing".equals(docClass)) {
            this.contentType = "application/vnd.sun.xml.draw";
            return;
        } else if ("presentation".equals(docClass)) {
            this.contentType = "application/vnd.sun.xml.impress";
            // If we didn't get a title from the metadata, 
            // let's use the first text box
            if (this.title == null) {
                Node headingElt =
                    DOMUtils.findElement(
                        root,
                        "http://openoffice.org/2000/text",
                        "p");
                if (headingElt != null) {
                    this.setTitle(DOMUtils.textContent(headingElt));
                }
            }
            return;
        } else if ("chart".equals(docClass)) {
            this.contentType = "application/vnd.sun.xml.calc";
            // XXX is this OK?
            return;
        }
    }

    private void determineTypeFromExtension() {
        int dotIndex = this.fileName.lastIndexOf(".");
        if (dotIndex < 1) {
            return;
        }
        String ext = this.fileName.substring(dotIndex + 1).toLowerCase();
        int i = Arrays.binarySearch(extensions, ext);
        if (i < 0) {
            return;
        }
        this.contentType = extensionTypes[i];
        this.guessedContentType = true;
    }

    private StringBuffer readToStringBuffer() throws IOException {
        int c;
        StringBuffer sb = new StringBuffer((int)this.file.length());
        BufferedInputStream in =
            new BufferedInputStream(new FileInputStream(this.file));
        try {
            while ((c = in.read()) != -1) {
                sb.append((char)c);
            }
        } finally {
            in.close();
        }
        return sb;
    }

    private void extractMetaFromHTML() throws IOException {
        try {
            boolean reparse = false;
            XMLReader tagSoup = SAXUtils.newTagSoupXMLReader();
            InputSource is = new InputSource(new FileInputStream(this.file));
            is.setSystemId("file:///foo");
            is.setEncoding(this.encoding);
            DomConsumer builder = DOMUtils.newDomConsumer();
            tagSoup.setContentHandler(builder.getContentHandler());
            tagSoup.parse(is);
            this.doc = builder.getDocument();
            Element root = doc.getDocumentElement();

            NodeList nl =
                root.getElementsByTagNameNS(
                    "http://www.w3.org/1999/xhtml",
                    "meta");
            int len = nl.getLength();
            for (int i = 0; i < len; i++) {
                Element meta = (Element)nl.item(i);
                if ("content-type"
                    .equalsIgnoreCase(meta.getAttribute("httpequiv"))) {
                    String enc = meta.getAttribute("content");
                    if (enc == null) {
                        break;
                    }
                    enc = enc.toLowerCase();
                    Matcher m = charsetPat.matcher(enc);
                    if(m.matches()) {
						enc = m.group(1);
						if (enc == null) {
							break;
						}                    	
                    } else {
                    	break;
                    }
                    if (!(enc.startsWith("utf-")
                        || enc.equals("iso-8859-1")
                        || enc.equals("windows-1252")
                        || enc.equals("us-ascii"))) {
                        this.encoding = enc;
                        reparse = true;
                        break;
                    }

                }
            }

            if (reparse) {
            	is.setByteStream(new FileInputStream(this.file));
                is.setEncoding(this.encoding);
                tagSoup.parse(is);
                this.doc = builder.getDocument();
                root = doc.getDocumentElement();
            }

            loop : for (
                Node n = root.getFirstChild();
                    n != null;
                    n = n.getNextSibling()) {
                if ("http://www.w3.org/1999/xhtml".equals(n.getNamespaceURI())
                    && "head".equals(n.getLocalName())) {
                    for (Node m = n.getFirstChild();
                        m != null;
                        m = m.getNextSibling()) {
                        if ("http://www.w3.org/1999/xhtml"
                            .equals(m.getNamespaceURI())
                            && "title".equals(m.getLocalName())) {
                            this.setTitle(DOMUtils.textContent(m));
                            break loop;
                        }
                    }
                }
            }
        } catch (Exception e) {
System.err.println(e);
        }
    }

    private void extractMetaFromPDF() throws IOException {
        try {
            PDFReader reader = new PDFReader(this.file);
            Dict info = reader.getInfo();
            this.setTitle(reader.getObject(info.get("Title")).toString());
        } catch (Exception e) {

        }
    }

    private String normalizeWhiteSpace(String text) {
        StringBuffer buf = new StringBuffer(text.length());
        boolean lastIsWhitespace = true;
        for (int i = 0; i < text.length(); i++) {
            char c = text.charAt(i);
            if (c == ' ' || c == '\t' || c == '\n' || c == '\r') {
                if (!lastIsWhitespace) {
                    buf.append(' ');
                    lastIsWhitespace = true;
                }
            } else {
                buf.append(c);
                lastIsWhitespace = false;
            }
        }
        if (buf.charAt(buf.length() - 1) == ' ') {
            buf.deleteCharAt(buf.length() - 1);
        }
        return buf.toString();
    }

    private void setTitle(String title) {
        String collapsed = this.normalizeWhiteSpace(title);
        if ("".equals(collapsed)) {
            this.title = null;
        } else {
            this.title = collapsed;
        }
    }

    /** 
     * Getter for property contentType.
     * @return Value of property contentType.
     */
    public java.lang.String getContentType() {
        return this.contentType;
    }

    /** 
     * Getter for property encoding
     * @return Value of property encoding.
     */
    public java.lang.String getEncoding() {
        return this.encoding;
    }

    /** 
     * Getter for property guessedContentType. If guessedContentType
     * is true, the content type was derived from the file name and it
     * could be wrong. If guessedContentType is false, the content
     * type was derived from the magic bytes of the file and you can
     * trust on that information.
     * @return the value of guessedContentType field
     */
    public boolean getGuessedContentType() {
        return this.guessedContentType;
    }

    public java.lang.String getTitle() {
        return this.title;
    }

    public Document getDOM() {
        return this.doc;
    }

    public static String[] getMimeTypes() {
        return mimeTypes;
    }

    public static String[] getMimeTypeNames() {
        return mimeTypeNames;
    }
}
