/*
 * Copyright (c) 2003, 2004 Henri Sivonen
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 */

package fi.iki.hsivonen.xml;
import gnu.xml.pipeline.EventConsumer;
import gnu.xml.pipeline.EventFilter;

import java.util.Arrays;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 *
 * @author  hsivonen
 */
public class XHTMLCruftDropper extends EventFilter {

    private static final String[] normalAttrs =
        {
            "accesskey",
            "cite",
            "class",
            "datetime",
            "dir",
            "id",
            "lang",
            "onblur",
            "onclick",
            "ondblclick",
            "onfocus",
            "onkeydown",
            "onkeypress",
            "onkeyup",
            "onmousedown",
            "onmousemove",
            "onmouseout",
            "onmouseup",
            "style",
            "tabindex",
            "title" };

    private static final String[] normalElts =
        {
            "abbr",
            "acronym",
            "address",
            "b",
            "big",
            "blockquote",
            "br",
            "caption",
            "cite",
            "code",
            "del",
            "dfn",
            "div",
            "dl",
            "em",
            "fieldset",
            "h1",
            "h2",
            "h3",
            "h4",
            "h5",
            "h6",
            "hr",
            "i",
            "input",
            "ins",
            "kbd",
            "noscript",
            "ol",
            "p",
            "pre",
            "q",
            "samp",
            "small",
            "span",
            "strong",
            "sub",
            "sup",
            "tt",
            "ul",
            "var" };

    private static final String[] specialElts =
        {
            "a",
            "bdo",
            "body",
            "button",
            "col",
            "colgroup",
            "form",
            "head",
            "html",
            "img",
            "label",
            "link",
            "map",
            "meta",
            "object",
            "script",
            "script",
            "select",
            "style",
            "table",
            "tbody",
            "td",
            "textarea",
            "tfoot",
            "th",
            "thead",
            "title",
            "tr" };

    private static final String[] specialAttrs =
        {
            "abbr",
            "accept",
            "accept-charset",
            "accesskey",
            "action",
            "align",
            "alt",
            "archive",
            "axis",
            "border",
            "cellpadding",
            "char",
            "charoff",
            "charset",
            "checked",
            "classid",
            "codebase",
            "cols",
            "colspan",
            "content",
            "coords",
            "data",
            "declare",
            "defer",
            "dir",
            "disabled",
            "enctype",
            "for",
            "frame",
            "headers",
            "height",
            "href",
            "hreflang",
            "http-equiv",
            "id",
            "ismap",
            "label",
            "longdesc",
            "maxlength",
            "media",
            "method",
            "multiple",
            "name",
            "nohref",
            "onblur",
            "onchange",
            "onchange",
            "onfocus",
            "onload",
            "onreset",
            "onselect",
            "onselect",
            "onsubmit",
            "onunload",
            "readonly",
            "rel",
            "rev",
            "rows",
            "rowspan",
            "rules",
            "scheme",
            "scope",
            "selected",
            "shape",
            "size",
            "span",
            "src",
            "standby",
            "summary",
            "tabindex",
            "type",
            "usemap",
            "valign",
            "value",
            "valuetype",
            "width" };

    /** Creates a new instance of XHTMLStricter */
    public XHTMLCruftDropper(EventConsumer next) {
        super(next);
    }

    public void endElement(String uri, String local, String qname)
        throws SAXException {
        if ("http://www.w3.org/1999/xhtml".equals(uri)
            && (Arrays.binarySearch(normalElts, local) >= 0
                || Arrays.binarySearch(specialElts, local) >= 0)) {
            super.endElement(uri, local, qname);
        }
    }

    public void ignorableWhitespace(char[] values, int param, int param2) {
    }

    public void skippedEntity(String str) throws SAXException {
        throw new SAXException("enresolved entity");
    }

    public void startElement(
        String uri,
        String local,
        String qname,
        Attributes attributes)
        throws SAXException {
        if ("http://www.w3.org/1999/xhtml".equals(uri)) {
            if (Arrays.binarySearch(normalElts, local) >= 0) {
                AttributesImpl attrs = new AttributesImpl(attributes);
                int i = 0;
                String attrLocal;
                String attrNS;
                while (i < attrs.getLength()) {
                    attrLocal = attrs.getLocalName(i);
                    attrNS = attrs.getURI(i);
                    if (("lang".equals(attrLocal)
                        && "http://www.w3.org/XML/1998/namespace".equals(attrNS))
                        || ("space".equals(attrLocal)
                            && "http://www.w3.org/XML/1998/namespace".equals(
                                attrNS))
                        || (Arrays.binarySearch(normalAttrs, attrLocal) >= 0
                            && ("".equals(attrNS)
                                || "http://www.w3.org/1999/xhtml".equals(
                                    attrNS)))) {
                        i++;
                    } else {
                        attrs.removeAttribute(i);
                    }
                }
                super.startElement(uri, local, qname, attrs);
            } else if (Arrays.binarySearch(specialElts, local) >= 0) {
                AttributesImpl attrs = new AttributesImpl(attributes);
                int i = 0;
                String attrLocal;
                String attrNS;
                while (i < attrs.getLength()) {
                    attrLocal = attrs.getLocalName(i);
                    attrNS = attrs.getURI(i);
                    if (("lang".equals(attrLocal)
                        && "http://www.w3.org/XML/1998/namespace".equals(attrNS))
                        || ("space".equals(attrLocal)
                            && "http://www.w3.org/XML/1998/namespace".equals(
                                attrNS))
                        || ((Arrays.binarySearch(normalAttrs, attrLocal) >= 0
                            || Arrays.binarySearch(specialAttrs, attrLocal) >= 0)
                            && ("".equals(attrNS)
                                || "http://www.w3.org/1999/xhtml".equals(
                                    attrNS)))) {
                        i++;
                    } else {
                        attrs.removeAttribute(i);
                    }
                }
                super.startElement(uri, local, qname, attrs);
            }
        }
    }
}
