/*
 * Copyright (c) 2003, 2004 Jussi Koskela
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package fi.iki.hsivonen.webcache.front;

import java.io.File;

import org.mortbay.http.HttpContext;
import org.mortbay.http.HttpHandler;
import org.mortbay.http.HttpServer;
import org.mortbay.http.NCSARequestLog;
import org.mortbay.http.SocketListener;
import org.mortbay.http.Version;
import org.mortbay.util.Code;

public class FrontServer {

    /** Do we print the debug information */
    private static boolean DEBUG = false;

    /** The name of the front server configuration file */
    public static final String CONFIGURATION_FILE = "cms-cache.conf";

    /**
     * Prints the given debug string.
     * @param debug the debug string
     */    
    public static void debug(String debug) {
        if (DEBUG) {
            System.out.println(debug);
        }
    }

    /** Program main entry point */
    public static void main (String[] args) throws Exception {        
        DEBUG = (System.getProperty("Debug") != null);

        try {
            FrontServer.debug("Starting WebCMS Front Server");

            // Load and verify the front server configuration
            FrontServerConfiguration fsg = 
                new FrontServerConfiguration(CONFIGURATION_FILE);

            // Use the content server as a proxy
            System.getProperties().put("http.proxyHost", 
                                       fsg.getContentServerHost());
            System.getProperties().put("http.proxyPort", 
                                       Integer.toString(fsg.getContentServerPort()));


            // Don't include os/java version information to the Server field
            Version.__VersionDetail = fsg.getServerTag();
            Code.setDebug(false);

            File directory = new File(fsg.getCacheDirectory());
            directory.mkdirs();

            // Create the server
            HttpServer server = new HttpServer();
	
            // Create the client listener
            SocketListener clientListener = new SocketListener();
            clientListener.setPort(fsg.getServerPort());
            clientListener.setMinThreads(fsg.getMinThreads());
            clientListener.setMaxThreads(fsg.getMaxThreads());
            server.addListener(clientListener);

            // Create the invalidator listener
            SocketListener invalidatorListener = new SocketListener();
            invalidatorListener.setPort(fsg.getInvalidatorPort());
            invalidatorListener.setMinThreads(1);
            invalidatorListener.setMaxThreads(4);
            server.addListener(invalidatorListener);

            // Create a root context
            HttpContext context = new HttpContext();
            context.setContextPath("/");
            context.setResourceBase(fsg.getCacheDirectory());
            context.setWelcomeFiles(null);
            server.addContext(context);

            // Create a cache handler
            HttpHandler cacheHandler = 
                new CacheHandler(directory, 
                                 fsg.getMaxDiskUsage(), 
                                 fsg.getMaxCacheFiles(),
                                 fsg.getServerPort(),
                                 fsg.getInvalidatorPort());
            context.addHandler(cacheHandler);


            // Create the logger
            if (fsg.getLogging()) {
                NCSARequestLog requestLog = 
                    new NCSARequestLog(fsg.getLogFile());
                requestLog.setAppend(true);
                requestLog.setBuffered(false);
                server.setRequestLog(requestLog);
            }

            // Start the http server
            server.start();

        } catch (InvalidConfigurationException ice) {
            System.out.println(ice.getMessage());
            System.exit(0);
        } catch (Exception e) {
            System.out.println(e);
            System.exit(0);
        }
    }
}
